#include <visa.h>
#include <ansi_c.h>
#include <utility.h>
#include <formatio.h>
#include "tktds3xx.h"

#define TKTDS3xx_REVISION     "Rev 1.0, 9/95, CVI 3.1" /*  Instrument driver revision */
#define BUFFER_SIZE         512L         /* File I/O buffer size */
#define WAVE_BUFFER_SIZE    4096L
    
/*= Tektronix TKTDS3xx Programmable Oscilloscope ==========================*/
/* LabWindows/CVI 3.1 Instrument Driver                                    */
/* Original Release: NOV., 1993                                            */
/* By: Ed Lingel, Tektronix, Inc.                                          */
/*                                                                         */                           
/* Modification History:                                                   */
/*                                                                         */                           
/*      April 1994 - Changed the GPIB Library calls so that they check     */
/*                   for the error bit being set to indicate an error      */
/*                   (& 0x8000) instead of checking for a negative         */
/*                   return value.                                         */
/*                                                                         */                           
/*                   Modified by:   JRO, NI Austin, Texas                  */
/*                   Phone (800) 433-3488  Fax (512) 794-5678              */
/*                                                                         */
/*      June 1995 -  Modified the LabWindows/CVI instrument driver to use  */
/*                   VISA I/O calls.                                       */
/*                                                                         */                           
/*                   Modified by:   VG, NI Austin, TX.                     */
/*                                                                         */                           
/*=========================================================================*/
 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

static ViString acquireMode[4] = {"SAMPLE", "PEAKDETECT", "ENVELOPE", "AVERAGE"};    
static ViString acquireState[2] = {"ON", "OFF"};
static ViString acquireStopAfter[2] = {"RUNSTOP", "SEQUENCE"};       
static ViString displayFormat[2] = { "XY", "YT"};
static ViString displayGraticule[2] = {"FULL", "FRAME"};
static ViString displayIntensity[2] = {"DIM", "BRIGHT"};
static ViString displayStyle[4] = {"DOTS", "VECTORS", "ACCUMVECTORS", "ACCUMDOTS"};
static ViString hardcopyFormat[11] = {"BMP", "DESKJET", "DPU411", "DPU412", "EPSIMAGE"
                                    , "EPSON", "INTERLEAF", "LASERJET", "PCX", "THINKJET", "TIFF"};
static ViString hardcopyLayout[2] = { "LANDSCAPE", "PORTRAIT"};
static ViString hardcopyPort[3] = {"GPIB", "CENTRONICS", "RS232"};
static ViString horizMode[3] = {  "MAIN", "INTENSIFIED", "DELAYED"};
static ViString mathFunc[3] = {"CH1 + CH2", "CH1 - CH2", "CH2 - CH1"};
static ViString measMethod[2] = {"MINMAX", "HISTOGRAM"};
static ViString measName[21] = {"AMPLITUDE", "BURST", "CMEAN", "CRMS", "FALL", "FREQUENCY"
                                , "HIGH", "LOW", "MAXIMUM", "MEAN", "MINIMUM", "NDUTY"
                                , "NOVERSHOOT", "NWIDTH", "PDUTY", "PERIOD", "PK2PK"
                                , "POVERSHOOT", "PWIDTH", "RISE", "RMS"};
static ViString measRefUnits[2] = {"ABSOLUTE", "PERCENT"};
static ViString offOn[2] = {"OFF", "ON"};
static ViString trigLevel[4] = {":LEVEL ", ":LEVEL TTL", ":LEVEL ECL", " SETLEVEL"};
static ViString trigCoupling[5] = {"DC", "AC", "HFREJ", "LFREJ", "NOISEREJ"};
static ViString trigMode[2] = {"AUTO", "NORMAL"};
static ViString trigSlope[2] = {"RISE", "FALL"};
static ViString trigSource[6] = {"","CH1", "CH2", "EXT", "EXT10", "LINE"};
static ViString vertBandwidth[2] = {"FULL", "TWENTY"};
static ViString vertChannel[3] = {"", "CH1", "CH2"};
static ViString vertCoup[3] = {"DC", "AC", "GND"};
static ViString vidtrigField[3] = {"FIELD1", "FIELD2", "ANY"};
static ViString waveformSource[6] = {"", "CH1", "CH2", "REF1", "REF2", "MATH1"}; 

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean TKTDS3xx_invalidViBooleanRange (ViBoolean val);
ViBoolean TKTDS3xx_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean TKTDS3xx_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean TKTDS3xx_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean TKTDS3xx_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean TKTDS3xx_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean TKTDS3xx_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus TKTDS3xx_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus TKTDS3xx_readToFile (ViSession instrSession, ViString filename, ViUInt32 readBytes, ViPUInt32 retCount);
ViStatus TKTDS3xx_writeFromFile (ViSession instrSession, ViString filename, ViUInt32 writeBytes, ViPUInt32 retCount);
ViStatus TKTDS3xx_defaultInstrSetup (ViSession openInstrSession);

ViStatus TKTDS3xx_wvfmSelected (ViSession instrSession, ViInt16 source);
ViStatus TKTDS3xx_poll (ViSession instrSession, ViUInt16 *response);
ViStatus TKTDS3xx_waitOpc (ViSession instrSession);
ViStatus TKTDS3xx_clearOpc (ViSession instrSession);
ViBoolean TKTDS3xx_fileOnDisk (ViChar *file_name);
ViStatus TKTDS3xx_checkInstrStatus (ViSession instrSession);
                                                              
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* TKTDS3xx_stringValPair is used in the TKTDS3xx_errorMessage function            */
/* TKTDS3xx_statusDataRanges is used to track session dependent status & ranges  */
/*===========================================================================*/
typedef struct  TKTDS3xx_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}   TKTDS3xx_tStringValPair;

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_init (ViRsrc resourceName, ViBoolean IDQuery, ViBoolean resetDevice,
                      ViSession *instrSession)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    if ((TKTDS3xx_status = viOpenDefaultRM (&rmSession)) < 0)
        return TKTDS3xx_status;

    if ((TKTDS3xx_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return TKTDS3xx_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/

    if ((TKTDS3xx_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);
    if ((TKTDS3xx_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);
    if ((TKTDS3xx_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);
    if ((TKTDS3xx_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);

    /*- Identification Query ------------------------------------------------*/

    if (IDQuery) {
        if ((TKTDS3xx_status = viWrite (*instrSession, "*IDN?", 5, &retCnt)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);
        if ((TKTDS3xx_status = viRead (*instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return TKTDS3xx_status;

        Scan (rdBuffer, "TEKTRONIX,TDS 3");
        if (NumFmtdBytes () != 15) 
            return TKTDS3xx_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }

    /*- Reset instrument ----------------------------------------------------*/

    if (resetDevice) {
        if ((TKTDS3xx_status = TKTDS3xx_reset (*instrSession)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);
    }       
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((TKTDS3xx_status = TKTDS3xx_defaultInstrSetup (*instrSession)) < 0)
            return TKTDS3xx_initCleanUp (rmSession, instrSession, TKTDS3xx_status);
        
    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Waveform Display                                */
/* Description:  This function controls the display of waveforms           */
/*               (turns them on or off).                                   */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configWaveformDisplay (ViSession instrSession,
                                       ViInt16 displaySource,
                                       ViBoolean displayOnOff)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViInt16Range (displaySource, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViBooleanRange (displayOnOff))
        return VI_ERROR_PARAMETER3;

    if (displaySource <= 2)
        Fmt (wrtBuf, "%s<SELECT:CH%d[b2] %d[b2];", displaySource, displayOnOff);
    else if (displaySource <= 4)
        Fmt (wrtBuf, "%s<SELECT:REF%d[b2] %d[b2];", displaySource - 2, displayOnOff);
    else
        Fmt (wrtBuf, "%s<SELECT:MATH1 %d[b2];", displayOnOff);

    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Auto Setup                                                    */
/* Purpose:  This function auto configures the instrument.                 */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_AutoSetup (ViSession instrSession)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if ((TKTDS3xx_status = viWrite (instrSession, "AUTOS EXEC;", 11, &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Vertical                                        */
/* Description:  This function controls the vertical settings of the       */
/*               instrument.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configVertical (ViSession instrSession, ViInt16 source,
                                ViReal64 voltsDiv, ViReal64 offset,
                                ViReal64 position, ViInt16 coupling,
                                ViBoolean bandwidth, ViBoolean invert)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (source, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViReal64Range (voltsDiv, 0.002, 10.0))
        return VI_ERROR_PARAMETER3;
    if (voltsDiv < 0.1)  {
        if (TKTDS3xx_invalidViReal64Range (offset, -1.0, 1.0))
            return VI_ERROR_PARAMETER4;
    }
    else if (voltsDiv < 1.0)  {
        if (TKTDS3xx_invalidViReal64Range (offset, -10.0, 10.0))
            return VI_ERROR_PARAMETER4;
    }
    else if (TKTDS3xx_invalidViReal64Range (offset, -100.0, 100.0))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViReal64Range (position, -5.0, 5.0))
        return VI_ERROR_PARAMETER5;
    if (TKTDS3xx_invalidViInt16Range (coupling, 0, 2))
        return VI_ERROR_PARAMETER6;
    if (TKTDS3xx_invalidViBooleanRange (bandwidth))
        return VI_ERROR_PARAMETER7;
    if (TKTDS3xx_invalidViBooleanRange (invert))
        return VI_ERROR_PARAMETER8;

    Fmt (wrtBuf, "%s<%s:SCALE %f[ep3];OFFSET %f[p3];POSITION %f[p3];COUPLING %s;BANDWIDTH %s;",
            vertChannel[source], voltsDiv, offset, position, vertCoup[coupling],
            vertBandwidth[bandwidth]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<%s:INVERT %d[b2];", vertChannel[source], invert);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Horizontal                                      */
/* Description:  This function controls the horizontal parameters of the   */
/*               instrument.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configHorizontal (ViSession instrSession, ViInt16 timeBase,
                                  ViReal64 mainScale, ViInt16 position,
                                  ViReal64 delayedScale)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViInt16Range (timeBase, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViReal64Range (mainScale, 2.5E-9, 5.0))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViInt16Range (position, 0, 100) != 0)
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViReal64Range (delayedScale, 2.5E-9, mainScale))
        return VI_ERROR_PARAMETER5;
    
    Fmt (wrtBuf, "%s<HORIZONTAL:MODE %s;POSITION %d[b2];", horizMode[timeBase],
            position);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<HORIZONTAL:MAIN:SCALE %f[ep3];", mainScale);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<HORIZONTAL:DELAY:SCALE %f[ep3];", delayedScale);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Horizontal Delay                                */
/* Description:  This function sets the delay mode (Runs After or Triggered*/
/*               After) and the delay time.                                */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configHorizontalDelay (ViSession instrSession,
                                       ViReal64 delayTime)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViReal64Range (delayTime, 0.00000001, 50.0))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<HORIZONTAL:DELAY:TIME:RUNSAFTER %f[ep9];", delayTime);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}
           
/*=========================================================================*/
/* Function:     Trigger Mode/Holdoff/Pos                                  */
/* Description:  This function sets the basic trigger controls.            */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configMainTrigger (ViSession instrSession, ViBoolean mode,
                                   ViReal64 holdoff, ViInt16 position,
                                   ViInt16 userDefinedPos)
{                                   
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViChar positionBuffer[15];

    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViBooleanRange (mode))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViReal64Range (holdoff, 0.0000005, 10.0))
        return VI_ERROR_PARAMETER3;
    if ((position != 0) && (position != 10) && (position != 50) &&
            (position != 90)) {
        return VI_ERROR_PARAMETER4;
    }
    if (TKTDS3xx_invalidViInt16Range (userDefinedPos, 0, 100))
        return VI_ERROR_PARAMETER5;

    if (position == 0)
        Fmt (positionBuffer, "%s<%d[b2]", userDefinedPos);
    else
        Fmt (positionBuffer, "%s<%d[b2]", position);

    Fmt (wrtBuf, "%s<TRIGGER:MAIN:MODE %s;HOLDOFF:VALUE %f;",
            trigMode[mode], holdoff);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<HORIZONTAL:TRIGGER:POSITION %s;", positionBuffer);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}
    
/*=========================================================================*/
/* Function:     Configure Edge Trigger                                    */
/* Description:  This function controls the edge trigger parameters.       */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configTriggerEdge (ViSession instrSession, ViInt16 source,
                                   ViInt16 coupling, ViBoolean slope, ViInt16 level,
                                   ViReal64 userDefinedLevel)
{           
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViChar trigLevBuff[15];

    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (coupling, 0, 4))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViBooleanRange (slope))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViInt16Range (level, 0, 3))
        return VI_ERROR_PARAMETER5;
    if (TKTDS3xx_invalidViReal64Range (userDefinedLevel, -120.0, 120.0))
        return VI_ERROR_PARAMETER6;
    
    if (level == 0)
        Fmt (trigLevBuff, "%s<%s%f", trigLevel[level], userDefinedLevel);
    else
        Fmt (trigLevBuff, "%s<%s", trigLevel[level]);

    Fmt (wrtBuf, "%s<TRIGGER:MAIN:TYPE EDGE;");
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<TRIGGER:MAIN:EDGE:SOURCE %s;COUPLING %s;SLOPE %s;",
            trigSource[source], trigCoupling[coupling], trigSlope[slope]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<TRIGGER:MAIN%s;", trigLevBuff);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Video Trigger                                   */
/* Description:  This function controls the video trigger parameters       */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configTriggerVideo (ViSession instrSession, ViInt16 source,
                                    ViInt16 field, ViBoolean triggerMode,
                                    ViReal64 holdoff)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (field, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViBooleanRange (triggerMode))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViReal64Range (holdoff, 500e-9, 10.0))
        return VI_ERROR_PARAMETER5;
           
    Fmt (wrtBuf, "%s<TRIGGER:MAIN:TYPE VIDEO;VIDEO:SOURCE %s;",
            trigSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<TRIGGER:MAIN:VIDEO:FIELD %s;", vidtrigField[field]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<TRIGGER:MAIN:MODE %s;", trigMode[triggerMode]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<TRIGGER:MAIN:VIDEO:HOLDOFF:VALUE %f;", holdoff);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Acquisition                                     */
/* Description:  This function controls the acquisition parameters.        */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configAcquire (ViSession instrSession, ViInt16 mode,
                               ViBoolean stopCondition, ViInt16 envelopeAcq,
                               ViInt16 averageAcq)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 i = 0, val = 0;

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (mode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (envelopeAcq, 0, 256))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViInt16Range (averageAcq, 2, 256))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViBooleanRange (stopCondition))
        return VI_ERROR_PARAMETER5;

    /* Check envelopeAcq & averageAcq for valid discrete values */
    if (envelopeAcq) {
        val = 0;
        for (i = 1; i <= 8; i++)
            if (envelopeAcq == pow (2, i))
                val = 1;
        if (!val) {
            TKTDS3xx_status = VI_ERROR_PARAMETER3;
            return TKTDS3xx_status;
        }
    }
    val = 0;
    for (i = 1; i <= 8; i++)
        if (averageAcq == pow (2, i))
            val = 1;
    if (!val) {
        TKTDS3xx_status = VI_ERROR_PARAMETER4;
        return TKTDS3xx_status;
    }

    Fmt (wrtBuf, "%s<ACQUIRE:MODE %s;STOPAFTER %s;", acquireMode[mode],
            acquireStopAfter[stopCondition]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<ACQUIRE:NUMENV %d[b2];NUMAVG %d[b2];", envelopeAcq, averageAcq);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Math Waveforms                                  */
/* Description:  This function defines a math waveform.                    */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configMath (ViSession instrSession, ViInt16 mathFunction)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (mathFunction, 0, 2))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<MATH1:DEFINE \"%s\";", mathFunc[mathFunction]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Setup Measurements                                        */
/* Description:  This function sets up measurement parameters for the      */
/*               instrument.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configMeasurement (ViSession instrSession,
                                   ViBoolean highLowSetup, ViBoolean gating,
                                   ViBoolean units, ViReal64 highReference,
                                   ViReal64 midReference, ViReal64 lowReference)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViBooleanRange (highLowSetup))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViBooleanRange (gating))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViBooleanRange (units))
        return VI_ERROR_PARAMETER4;

    if (units != 0)  {
        /*   units in percent   */
        if (TKTDS3xx_invalidViReal64Range (highReference, 0.0, 100.0))
            return VI_ERROR_PARAMETER5;
        if (TKTDS3xx_invalidViReal64Range (midReference, 0.0, 100.0))
            return VI_ERROR_PARAMETER6;
        if (TKTDS3xx_invalidViReal64Range (lowReference, 0.0, 100.0))
            return VI_ERROR_PARAMETER7;
    }
    else  {
        /*   units in volts, large valid range   */
        if (TKTDS3xx_invalidViReal64Range (highReference,-1000000.0, 1000000.0))
            return VI_ERROR_PARAMETER5;
        if (TKTDS3xx_invalidViReal64Range (midReference,-1000000.0, 1000000.0))
            return VI_ERROR_PARAMETER6;
        if (TKTDS3xx_invalidViReal64Range (lowReference,-1000000.0, 1000000.0))
            return VI_ERROR_PARAMETER7;
    }
    Fmt (wrtBuf, "MEASUREMENT:METHOD %s;GATING %s;", measMethod[highLowSetup],
            offOn[gating]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "MEASUREMENT:REFLEVEL:METHOD %s;%s:HIGH %f;",
            measRefUnits[units], measRefUnits[units], highReference);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "MEASUREMENT:REFLEVEL:%s:MID %f;", measRefUnits[units],
            midReference);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "MEASUREMENT:REFLEVEL:%s:LOW %f;", measRefUnits[units],
            lowReference);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Force Trigger                                             */
/* Description:  This function forces a trigger.                           */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_forceTrig (ViSession instrSession)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    Fmt (wrtBuf, "%s<TRIGGER FORCE;");
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Acquisition Run                                               */
/* Purpose:  This function starts or stops the acquisition.                */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_acquisitionRun (ViSession instrSession, ViBoolean acquire)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    
    /*- Check input parameter ranges ----------------------------------------*/
           
    if (TKTDS3xx_invalidViBooleanRange (acquire))
        return VI_ERROR_PARAMETER2;
           
    Fmt (wrtBuf, "%s<ACQ:STATE %d[b2];", acquire);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Read Measurement                                          */
/* Description:  This function reads a measurement from the instrument.    */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readMeasurement (ViSession instrSession, ViInt16 measurement,
                                 ViInt16 source, ViReal64 *reading)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (measurement, 0, 20))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "MEASUREMENT:IMMED:SOURCE %s;", waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "MEASUREMENT:IMMED:TYPE %s;", measName[measurement]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "MEASUREMENT:IMMED:VALUE?;");
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (viScanf (instrSession, "%lf", reading) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;
 
    return TKTDS3xx_status;
}
 
/*=========================================================================*/
/* Function: Number Of Acquisitions                                        */
/* Purpose:  This function returns the number of acquisitions taken.       */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_numberAcq (ViSession instrSession, ViInt16 *acquisitionCount)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    if ((TKTDS3xx_status = viWrite (instrSession, "ACQ:NUMAC?;", 11, &retCnt)) < 0)
        return TKTDS3xx_status;

    if (viScanf (instrSession, "%hd", acquisitionCount) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;
    
    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Read Waveform to Array                                        */
/* Purpose:  This function reads a wvfm from the instrument into an array. */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readWvfmArray (ViSession instrSession, ViInt16 source,
                               ViInt16 start, ViInt16 stop,
                               ViReal64 waveformArray[], ViInt16 *returnPoints,
                               ViReal64 *XStart, ViReal64 *XIncrement)
{
    ViInt16 wvSize;
    ViInt16 index;
    ViInt16 bytesLeft;
    ViInt16 readCnt;
    ViInt16 i;
    ViInt16 offset;
    ViInt16 selected;
    ViReal64 ymult;
    ViReal64 ptO;
    ViReal64 yinc;
    ViReal64 yzero;
    ViReal64 yoff;
    ViInt16  wave[2048];

    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[WAVE_BUFFER_SIZE];

    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (start, 1, 1000))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViInt16Range (stop, 1, 1000))
        return VI_ERROR_PARAMETER4;
    if ((start > stop) || (stop - start + 1 > 8192))  {
        return VI_ERROR_PARAMETER4;
    }

    selected = TKTDS3xx_wvfmSelected (instrSession, source);
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<SEL:%s ON;:DAT:SOU %s;STAR %d[b2];STOP %d[b2];WID 2;:WFMP:ENC BIN;BN_F RI;PT_F Y;BYT_O LSB;",
            waveformSource[source], waveformSource[source], start, stop);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (!TKTDS3xx_wvfmSelected (instrSession, source))
        return VI_ERROR_REF_NOT_EXIST;

    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<:WFMP:%s:NR_P?;XIN?;YMU?;YOF?;YZE?;",
            waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = viRead (instrSession, rdBuf, 4096, &retCnt) < 0)
        return TKTDS3xx_status;
    if (Scan (rdBuf, "%s>%d[b2];%f;%f;%f;%f", &wvSize, XIncrement, &ymult, &yoff,
            &yzero) != 5)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if (TKTDS3xx_status = viWrite (instrSession, "CURV?;", 6, &retCnt) < 0)
        return TKTDS3xx_status;

    FillBytes (rdBuf, 0, WAVE_BUFFER_SIZE, 0);

    if (TKTDS3xx_status = viRead (instrSession, rdBuf, 2, &retCnt) < 0)
        return TKTDS3xx_status;
    if (Scan (rdBuf, "%s>#%d[b2]", &offset) != 1)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if (TKTDS3xx_status = viRead (instrSession, rdBuf, offset, &retCnt) < 0)
        return TKTDS3xx_status;
    bytesLeft = (stop - start + 1) * 2;
    index = 0;
    while (bytesLeft > 0)  {
        if (bytesLeft > 4096)
            readCnt = 2048;
        else
            readCnt = RoundRealToNearestInteger ((double)(bytesLeft / 2));
        if (TKTDS3xx_status = viRead (instrSession, rdBuf, readCnt * 2, &retCnt) < 0)
            return TKTDS3xx_status;
        bytesLeft -= readCnt * 2;
        if (Scan (rdBuf, "%*d[zb2o01]>%*d[b2]", readCnt, readCnt, wave) != 1)
            return VI_ERROR_INTERPRETING_RESPONSE;
        for (i = 0; i < readCnt; i++)
            waveformArray[i + index] = yzero + ((double)wave[i] - yoff) * ymult;
        index += readCnt;
    }
    *returnPoints = stop - start + 1;
    *XStart = *XIncrement * (double)(start - 1);

    if (!selected)  {
        Fmt (wrtBuf, "%s<SEL:%s OFF;", waveformSource[source]);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Store Waveform to file                                        */
/* Purpose:  This function saves a waveform to a file.                     */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_storeWvfmFile (ViSession instrSession, ViInt16 source,
                               ViChar filename[], ViBoolean overwrite)
{
    ViInt16 fileHandle;
    ViInt16 bytesLeft;
    ViInt32 bytesLeftLong;
    ViInt16 readCnt;
    ViInt16 i;
    ViInt16 selected;
    ViReal64 temp[5];
    ViChar fullFilename[80];
    
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[WAVE_BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViBooleanRange (overwrite))
        return VI_ERROR_PARAMETER4;
    
    Fmt (fullFilename, "%s<%s.wav", filename);
    if (!overwrite)
        if (TKTDS3xx_fileOnDisk (fullFilename) != 0)  {
            TKTDS3xx_status = VI_ERROR_PARAMETER3;
            return TKTDS3xx_status;
        }
    fileHandle = OpenFile (fullFilename, VAL_WRITE_ONLY, VAL_TRUNCATE,
                           VAL_BINARY);
    if (fileHandle ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_OPEN;
        return TKTDS3xx_status;
    }
    selected = TKTDS3xx_wvfmSelected (instrSession, source);
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<:SEL:%s ON;:DAT:SOU %s;STAR 1;STOP 1000;WID 2;:WFMP:ENC BIN;BN_F RI;PT_F Y;:WFMP:BYT_O LS""B",
            waveformSource[source], waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    /*  make sure wvfm selected  */
    if (!TKTDS3xx_wvfmSelected (instrSession, source))  {
        TKTDS3xx_status = VI_ERROR_REF_NOT_EXIST;
        return TKTDS3xx_status;
    }
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;
    Fmt (wrtBuf, "%s<:WFMP:%s:NR_P?;XIN?;YMU?;YOF?;YZE?", waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = viRead (instrSession, rdBuf, 4096, &retCnt) < 0)
        return TKTDS3xx_status;
    FmtFile (fileHandle, "%s<TDS3xx.WAVEDATA");
    if (Scan (rdBuf, "%s>%d[b2];%4f[x]", &bytesLeft, temp) != 2) 
        return VI_ERROR_INTERPRETING_RESPONSE;
    FmtFile (fileHandle, "%d[b2]<%d[b2]", bytesLeft);
    bytesLeftLong = 2L * (long)bytesLeft;
    FmtFile (fileHandle, "%4f<%4f", temp);

    Fmt (wrtBuf, "%s<HEADER ON;VERB ON;:DAT:SOUR %s;:WFMPR:%s?",
            waveformSource[source], waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = viRead (instrSession, rdBuf, 4096, &retCnt) < 0)
        return TKTDS3xx_status;
    FmtFile (fileHandle, "%d[b2]<%d[b2]", StringLength (rdBuf));
    FmtFile (fileHandle, "%s<%sDATA", rdBuf);
    if (TKTDS3xx_status = viWrite (instrSession, "HEADER OFF;VERB OFF;CURV?", 25, &retCnt) < 0)
        return TKTDS3xx_status;

    FillBytes (rdBuf, 0, WAVE_BUFFER_SIZE, 0);

    if (TKTDS3xx_status = viRead(instrSession, rdBuf, 2, &retCnt) < 0)
        return TKTDS3xx_status;
    if (Scan (rdBuf, "%s>#%d[b2]", &readCnt) != 1)
        return VI_ERROR_INTERPRETING_RESPONSE;
    if (TKTDS3xx_status = viRead(instrSession, rdBuf, readCnt, &retCnt) < 0)
        return TKTDS3xx_status;
    while (bytesLeftLong > 0L)  {
        if (bytesLeftLong > 4096L)
            readCnt = 4096;
        else
            readCnt = (ViInt16)bytesLeftLong;
        if (TKTDS3xx_status = viRead (instrSession, rdBuf, readCnt, &retCnt) < 0)
            return TKTDS3xx_status;
        i = FmtFile (fileHandle, "%s[w*t-]<%s[t-w*]", readCnt, readCnt, rdBuf);
        i = NumFmtdBytes ();
        bytesLeftLong -= (long)readCnt;
    }
    if (CloseFile (fileHandle) ==  -1)
        return VI_ERROR_FILE_CLOSE;
    if (!selected)  {
        Fmt (wrtBuf, "%s<SEL:%s OFF;", waveformSource[source]);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Read Waveform File                                            */
/* Purpose:  This function recalls part of a saved waveform to an array.   */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readWvfmFile (ViChar filename[], ViInt16 start,
                              ViInt16 stop, ViReal64 waveformArray[],
                              ViInt16 *returnPoints, ViReal64 *XStart,
                              ViReal64 *XIncrement)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[BUFFER_SIZE];
    ViChar fullFilename[80];
    ViInt16 fileHandle;
    ViInt16 wvSize;
    ViInt16 wordsLeft;
    ViInt16 readCnt;
    ViInt16 index;
    ViInt16 i;
    ViInt16 skip;
    ViInt16 error;
    ViInt32 readLoc;
    ViReal64 ymult;
    ViReal64 yof;
    ViReal64 yze;
    ViInt16 num;
    ViInt16 wave[2048];

    /*- Check input parameter ranges ----------------------------------------*/

    Fmt (fullFilename, "%s<%s.wav", filename);
    if (TKTDS3xx_fileOnDisk (fullFilename) != 1)  
         return VI_ERROR_PARAMETER1;
    fileHandle = OpenFile (fullFilename, 1, 0, 0);
    if (fileHandle ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_OPEN;
        return TKTDS3xx_status;
    }
    ScanFile (fileHandle, "%s[w15]>TDS3xx.WAVEDATA");
    if (NumFmtdBytes () != 15)  {
        TKTDS3xx_status = VI_ERROR_FILE_WRITE;
        return TKTDS3xx_status;
    }
    if (ScanFile (fileHandle, "%d[b2]>%d[b2]", &wvSize) != 1)
         return VI_ERROR_INTERPRETING_RESPONSE;
    if (TKTDS3xx_invalidViInt16Range (start, 1, wvSize))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (stop, 1, wvSize))
        return VI_ERROR_PARAMETER3;
    if (start > stop || stop - start + 1 > 8192)  {
        TKTDS3xx_status =  VI_ERROR_PARAMETER3;
        return TKTDS3xx_status;
    }
    if (ScanFile (fileHandle, "%4f>%f%f%f%f", XIncrement, &ymult, &yof, &yze) != 4)
        return VI_ERROR_INTERPRETING_RESPONSE;
    if (ScanFile (fileHandle, "%d[b2]>%d[b2]", &skip) != 1)
        return VI_ERROR_INTERPRETING_RESPONSE;
        
    SetFilePtr (fileHandle, (long)skip, 1);
    ScanFile (fileHandle, "%s[w4]>DATA");
    if (NumFmtdBytes () != 4)  {
        TKTDS3xx_status = VI_ERROR_FILE_WRITE;
        if (CloseFile (fileHandle) ==  -1)  {
            TKTDS3xx_status = VI_ERROR_FILE_CLOSE;
            return TKTDS3xx_status;
        }
        return TKTDS3xx_status;
    }
    wordsLeft = stop - start + 1;
    *returnPoints = wordsLeft;
    readLoc = (long)(start - 1) * 2L;
    SetFilePtr (fileHandle, readLoc, 1);
    index = 0;
    while (wordsLeft > 0)  {
        if (wordsLeft > 2048)
            readCnt = 2048;
        else
            readCnt = wordsLeft;
        if (ScanFile (fileHandle, "%*d[b2]>%*d[b2]", readCnt, readCnt, wave) != 1) {
            TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
            return TKTDS3xx_status;
        }
        wordsLeft -= readCnt;
        for (i = 0; i < readCnt; i++)
            waveformArray[i + index] = yze + ((double)wave[i] - yof) * ymult;
        index += readCnt;
    }
    *XStart = *XIncrement * (double)(start - 1);
    if (CloseFile (fileHandle) ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_CLOSE;
        return TKTDS3xx_status;
    }
    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Store Waveform to Memory                                      */
/* Purpose:  This function stores a waveform to the instrument memory.     */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_storeWvfmMemory (ViSession instrSession, ViInt16 source,
                                 ViInt16 destination)
{
    ViInt16 selected;
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (destination, 1, 2))
        return VI_ERROR_PARAMETER3;


    selected = TKTDS3xx_wvfmSelected (instrSession, source);
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<:SEL:%s ON;:DATA:SOURCE %s;", waveformSource[source],
            waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (!TKTDS3xx_wvfmSelected (instrSession, source))  {
        TKTDS3xx_status = VI_ERROR_REF_NOT_EXIST;
        return TKTDS3xx_status;
    }
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<:SAV:WAVE %s,REF%d[b2]", waveformSource[source], destination);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    /*  Check the Status Byte of the instrument for proper execution.  */
    if (TKTDS3xx_status = TKTDS3xx_checkInstrStatus (instrSession) < 0)
        return TKTDS3xx_status;

    if (!selected)  {
        Fmt (wrtBuf, "%s<SEL:%s OFF;", waveformSource[source]);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Read Waveform File To Instrument Memory                       */
/* Purpose:  This function copies a saved waveform (from file) to the      */
/*           instrument memory.                                            */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readWvfmMemory (ViSession instrSession, ViChar filename[],
                                ViInt16 destination)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[WAVE_BUFFER_SIZE];
    ViChar fullFilename[80];
    ViInt16 fileHandle;
    ViInt16 wvSize;
    ViInt16 readCnt;
    ViInt32 bytes_left;
    ViInt16 source;
    ViInt16 num;
    ViChar setupstr[5000];
    
    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViInt16Range (destination, 1, 2))
        return VI_ERROR_PARAMETER3;

    Fmt (fullFilename, "%s<%s.wav", filename);
    if (TKTDS3xx_fileOnDisk (fullFilename) != 1)  {
        TKTDS3xx_status =  VI_ERROR_PARAMETER2;
        return TKTDS3xx_status;
    }
    fileHandle = OpenFile (fullFilename, 1, 0, 0);
    if (fileHandle ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_OPEN;
        return TKTDS3xx_status;
    }
    ScanFile (fileHandle, "%s[w15]>TDS3xx.WAVEDATA");
    if (NumFmtdBytes () != 15)  {
        TKTDS3xx_status = VI_ERROR_FILE_WRITE;
        return TKTDS3xx_status;
    }
    if (ScanFile (fileHandle, "%d[b2]>%d[b2]", &wvSize) != 1) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }

/*    Fmt (wrtBuf, "%s<DAT:DEST REF%d[b2];:ALLO:WAVE:REF%d[b2] %d[b2];:DAT:ENC SRI;WID 2;STAR 1;",
            destination, destination, wvSize);*/
    Fmt (wrtBuf, "%s<DAT:DEST REF%d[b2];:DAT:ENC SRI;WID 2;STAR 1;", destination);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    /*  Check the Status Byte of the instrument for proper execution.  */
    if (TKTDS3xx_status = TKTDS3xx_checkInstrStatus (instrSession) < 0)
        return TKTDS3xx_status;

    SetFilePtr (fileHandle, 32L, 1);
    if (ScanFile (fileHandle, "%d[b2]>%d[b2]", &readCnt) != 1) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    if (ScanFile (fileHandle, "%s[w*t-]>%s[w*t-]", readCnt, readCnt, rdBuf) != 1) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }

    TKTDS3xx_status = viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_OFF);
    /*  disable sending END */
    if (TKTDS3xx_status = viWrite (instrSession, rdBuf, NumFmtdBytes (), &retCnt) < 0)
        return TKTDS3xx_status;
    ScanFile (fileHandle, "%s[w4]>DATA");
    if (NumFmtdBytes () != 4)  {
        TKTDS3xx_status = VI_ERROR_FILE_WRITE;
        return TKTDS3xx_status;
    }
    bytes_left = 2L * (long)wvSize;
    while (bytes_left > 0L)  {
        if (bytes_left > 4096L)
            readCnt = 4096;
        else
            readCnt = (ViInt16)bytes_left;
        ScanFile (fileHandle, "%s[w*t-]>%s[w*t-]", readCnt, readCnt, rdBuf);
        if (NumFmtdBytes () != readCnt)  {
            TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
            /*  error read from file */
            if (CloseFile (fileHandle) ==  -1)  {
                TKTDS3xx_status = VI_ERROR_FILE_CLOSE;
                return TKTDS3xx_status;
            }
            return TKTDS3xx_status;
        }
        Fmt(wrtBuf, "%s<%d[b2]", readCnt);
        num = NumFmtdBytes();
        Fmt(setupstr, "%s[t-]<:CURV #%d[b2]%s%s[w*t-]", num, wrtBuf, readCnt, rdBuf);
        bytes_left -= (long)readCnt;
        if (bytes_left <= 0L)
            TKTDS3xx_status = viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_ON);
            /*  enable sending END */
        num = readCnt + 8 + num;
        if (TKTDS3xx_status = viWrite (instrSession, setupstr, num, &retCnt) < 0)
            return TKTDS3xx_status;
    }
    if (CloseFile (fileHandle) ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_CLOSE;
        return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Read Waveform to Integer Array                                */
/* Purpose:  This function reads a waveform from the instrument into an    */
/*           integer array.                                                */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readWvfm2int (ViSession instrSession, ViInt16 source,
                              ViInt16 start, ViInt16 stop, ViInt16 waveformArray[],
                              ViInt16 *returnPoints, ViReal64 *XStart,
                              ViReal64 *XIncrement, ViReal64 *YGain,
                              ViReal64 *YOffset)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[WAVE_BUFFER_SIZE];
    ViInt16 wvSize;
    ViInt16 i;
    ViInt16 selected;
    ViInt16 offset;
    ViReal64 yof;

    /*- Check input parameter ranges ----------------------------------------*/
    
    if (TKTDS3xx_invalidViInt16Range (source, 1, 5))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (start, 1, 1000))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViInt16Range (stop, 1, 1000))
        return VI_ERROR_PARAMETER4;
    if (start > stop)
        return VI_ERROR_PARAMETER4;

    selected = TKTDS3xx_wvfmSelected (instrSession, source);
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<SEL:%s ON;:DAT:SOU %s;STAR %d[b2];STOP %d[b2];WID 2;:WFMP:ENC BIN;BN_F RI;PT_F Y;BYT_O LSB;",
            waveformSource[source], waveformSource[source], start, stop);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (!TKTDS3xx_wvfmSelected (instrSession, source))  {
        TKTDS3xx_status = VI_ERROR_REF_NOT_EXIST;
        return TKTDS3xx_status;
    }
    if (TKTDS3xx_status < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<:WFMP:%s:NR_P?;XIN?;YMU?;YOF?;YZE?;", waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = viRead (instrSession, rdBuf, 4096, &retCnt) < 0)
        return TKTDS3xx_status;
    if (Scan (rdBuf, "%s>%d[b2];%f;%f;%f;%f", &wvSize, XIncrement, YGain, &yof, YOffset) != 5)  {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    *YOffset -= *YGain * yof;

    if (TKTDS3xx_status = viWrite (instrSession, "CURV?;", 6, &retCnt) < 0)
        return TKTDS3xx_status;

    FillBytes (rdBuf, 0, WAVE_BUFFER_SIZE, 0);

    if (TKTDS3xx_status = viRead (instrSession, rdBuf, 2, &retCnt) < 0)
        return TKTDS3xx_status;
    if (Scan (rdBuf, "%s>#%d[b2]", &offset) != 1)  {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = viRead (instrSession, rdBuf, offset, &retCnt) < 0)
        return TKTDS3xx_status;

    *returnPoints = stop - start + 1;
    *XStart = *XIncrement * (double)(start - 1);

    /*  read waveform into integer array  */
    if (TKTDS3xx_status = viRead (instrSession, (ViBuf)waveformArray, (long)(*returnPoints * 2), &retCnt) < 0)
        return TKTDS3xx_status;
    if (!selected)  {
        Fmt (wrtBuf, "%s<SEL:%s OFF;", waveformSource[source]);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Read Waveform File to Integer Array                           */
/* Purpose:  This function recalls part of a saved waveform to an array.   */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readFile2int (ViChar filename[], ViInt16 start, ViInt16 stop,
                              ViInt16 waveformArray[], ViInt16 *returnPoints,
                              ViReal64 *XStart, ViReal64 *XIncrement,
                              ViReal64 *YGain, ViReal64 *YOffset)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[BUFFER_SIZE];
    ViChar fullFilename[80];
    ViInt16 fileHandle, wvSize, i, skip, error, readCt;
    ViInt32 readLoc;
    ViReal64 yof;
    ViInt16 num;

    /*- Check input parameter ranges ----------------------------------------*/

    Fmt (fullFilename, "%s<%s.wav", filename);
    if (TKTDS3xx_fileOnDisk (fullFilename) != 1)  {
        TKTDS3xx_status =  VI_ERROR_PARAMETER1;
        return TKTDS3xx_status;
    }
    fileHandle = OpenFile (fullFilename, 1, 0, 0);
    if (fileHandle ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_OPEN;
        return TKTDS3xx_status;
    }
    ScanFile (fileHandle, "%s[w15]>TDS3xx.WAVEDATA");
    if (NumFmtdBytes () != 15)  {
        TKTDS3xx_status = VI_ERROR_FILE_WRITE;
        return TKTDS3xx_status;
    }
    if (ScanFile (fileHandle, "%d[b2]>%d[b2]", &wvSize) != 1) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    if (TKTDS3xx_invalidViInt16Range (start, 1, wvSize))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (stop, 1, wvSize))
        return VI_ERROR_PARAMETER3;
    if (start > stop)  {
        TKTDS3xx_status =  VI_ERROR_PARAMETER4;
        return TKTDS3xx_status;
    }

    if (ScanFile (fileHandle, "%4f>%f%f%f%f", XIncrement, YGain, &yof, YOffset) != 4) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    *YOffset -= *YGain * yof;
    if (ScanFile (fileHandle, "%d[b2]>%d[b2]", &skip) != 1) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    SetFilePtr (fileHandle, (long)skip, 1);
    ScanFile (fileHandle, "%s[w4]>DATA");
    if (NumFmtdBytes () != 4)  {
        TKTDS3xx_status = VI_ERROR_FILE_WRITE;
        if (CloseFile (fileHandle) ==  -1)  {
            TKTDS3xx_status = VI_ERROR_FILE_CLOSE;
            return TKTDS3xx_status;
        }
        return TKTDS3xx_status;
    }
    *returnPoints = stop - start + 1;
    readCt = *returnPoints;
    readLoc = (long)(start - 1) * 2L;
    SetFilePtr (fileHandle, readLoc, 1);
    if (ScanFile (fileHandle, "%*d[b2]>%*d[b2]", readCt, readCt, waveformArray) != 1) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        return TKTDS3xx_status;
    }
    *XStart = *XIncrement * (double)(start - 1);

    if (CloseFile (fileHandle) ==  -1)  {
        TKTDS3xx_status = VI_ERROR_FILE_CLOSE;
        return TKTDS3xx_status;
    }
    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Integer Array to Double Array Waveform                        */
/* Purpose:  This function recalls part of a saved waveform to an array.   */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_int2realWvfm (ViInt16 integerWvfmArray[], ViInt16 arraySize,
                              ViInt16 start, ViInt16 stop, ViReal64 YGain,
                              ViReal64 YOffset, ViInt16 *returnPoints,
                              ViReal64 wvfmDoubleArray[])
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViByte rdBuf[BUFFER_SIZE];
    ViInt16 i, j;

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (arraySize, 1, 32767))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (start, 1, arraySize))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViInt16Range (stop, 1, arraySize))
        return VI_ERROR_PARAMETER4;
    if ((stop < start) || ((stop - start) >= 8192))  {
        TKTDS3xx_status =  VI_ERROR_PARAMETER4;
        return TKTDS3xx_status;
    }
    j = start - 1;
    *returnPoints = stop - start + 1;
    for (i = 0; i < *returnPoints; i++)
        wvfmDoubleArray[i] = (double)integerWvfmArray[j + i] * YGain + YOffset;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Configure Display                                         */
/* Description:  This function controls the screen display parameters.     */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_configDisplay (ViSession instrSession, ViInt16 style,
                               ViReal64 accumulateTime, ViInt16 overallIntensity,
                               ViBoolean textGratIntensity, ViBoolean wvfmIntensity,
                               ViInt16 contrast, ViBoolean format,
                               ViBoolean graticule, ViBoolean triggerDisplay)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (style, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViReal64Range (style, 0.0, 10.0))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViInt16Range (overallIntensity, 20, 100))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViBooleanRange (textGratIntensity))
        return VI_ERROR_PARAMETER5;
    if (TKTDS3xx_invalidViBooleanRange (wvfmIntensity))
        return VI_ERROR_PARAMETER6;
    if (TKTDS3xx_invalidViInt16Range (contrast, 100, 250))
        return VI_ERROR_PARAMETER7;
    if (TKTDS3xx_invalidViBooleanRange (format))
        return VI_ERROR_PARAMETER8;
    if (TKTDS3xx_invalidViBooleanRange (graticule))
        return VI_ERROR_PARAMETER9;
    if (TKTDS3xx_invalidViBooleanRange (triggerDisplay))
        return VI_ERROR_PARAMETER10;

    Fmt (wrtBuf, "%s<:DISPLAY:STYLE %s;", displayStyle[style]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (style > 1)  {
        /*  accumulated display mode   */
        Fmt (wrtBuf, "%s<DISPLAY:PERSISTENCE %f;", style);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    Fmt (wrtBuf, "%s<:DISPLAY:FORMAT %s;GRATICULE %s;TRIGT %d[b2];",
            displayFormat[format], displayGraticule[graticule],
            triggerDisplay);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    Fmt (wrtBuf, "%s<:DISPLAY:INTENSITY:OVERALL %d[b2];TEXT %s;WAVEFORM %s;CONTRAST %d[b2];",
            overallIntensity, displayIntensity[textGratIntensity],
            displayIntensity[wvfmIntensity], contrast);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Save Setup                                                */
/* Description:  This function saves the current instrument setup to an    */
/*               instrument setup memory or to a file.                     */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_saveSetup (ViSession instrSession, ViBoolean setupLocation,
                           ViChar fileName[], ViInt16 instrumentSetup,
                           ViBoolean overwriteFile)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt32 fileSize;
    
    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViBooleanRange (setupLocation))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (instrumentSetup, 1, 10))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViBooleanRange (overwriteFile))
        return VI_ERROR_PARAMETER5;
   
    if (!setupLocation)  {
        /*   save to scope   */
        Fmt (wrtBuf, "%s<:SAVE:SETUP %d[b2];", instrumentSetup);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    else  {
        /*   save to file   */
        if (GetFileInfo (fileName, &fileSize) != 0)
            if (!overwriteFile)  {
                TKTDS3xx_status = VI_ERROR_FILE_EXISTS;
                /*   file exists, cannot overwrite   */
                return TKTDS3xx_status;
            }
        Fmt (wrtBuf, "%s<*LRN?;");
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;

        if (TKTDS3xx_status = TKTDS3xx_readToFile (instrSession, fileName, BUFFER_SIZE, &retCnt) < 0)
            return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Recall Setup                                              */
/* Description:  This function recalls instrument setup information from an*/
/*               instrument setup memory or from a file.                   */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_recallSetup (ViSession instrSession, ViBoolean setupLocation,
                             ViInt16 instrumentLocation, ViChar fileName[])
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 fileHandle;
    ViInt16 n;
    ViInt32 fileSize;
    ViInt16 timeout;   /*  temporary storage for old time-out value  */

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViBooleanRange (setupLocation))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViInt16Range (instrumentLocation, 1, 10))
        return VI_ERROR_PARAMETER4;

    if (!setupLocation)  {
        /*   recall from scope   */
        Fmt (wrtBuf, "%s<*RCL %d[b2];", instrumentLocation);
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    else  {
        /*   recall from file   */
        if (!GetFileInfo (fileName, &fileSize))  {
            TKTDS3xx_status = VI_ERROR_FILE_OPEN;
            /*   unable to open file   */
            return TKTDS3xx_status;
        }
        
        if (TKTDS3xx_status = viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &timeout) < 0)
                return TKTDS3xx_status;
        if (TKTDS3xx_status = viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 100000) < 0)
                return TKTDS3xx_status;
        /*  set time-out to 100 seconds  */
        if (TKTDS3xx_status = TKTDS3xx_writeFromFile (instrSession, fileName, BUFFER_SIZE, &retCnt) < 0) {
            /*  reset time-out  */
            if (TKTDS3xx_status = viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout) < 0)
                return TKTDS3xx_status;
            return TKTDS3xx_status;
        }
    }
    /*  reset time-out  */
    if (TKTDS3xx_status = viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Hardcopy                                                  */
/* Description:  This function sets the instrument hardcopy parameters and */
/*               starts a hardcopy.                                        */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_hardcopy (ViSession instrSession, ViInt16 format,
                          ViBoolean layout, ViBoolean port, ViChar fileName[],
                          ViBoolean clearMenus)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];
    ViInt16 File_Handle;
    ViInt16 Record_Length;

    /*- Check input parameter ranges ----------------------------------------*/

    if (TKTDS3xx_invalidViInt16Range (format, 0, 10))
        return VI_ERROR_PARAMETER2;
    if (TKTDS3xx_invalidViBooleanRange (layout))
        return VI_ERROR_PARAMETER3;
    if (TKTDS3xx_invalidViBooleanRange (port))
        return VI_ERROR_PARAMETER4;
    if (TKTDS3xx_invalidViBooleanRange (clearMenus))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<HARDCOPY:FORMAT %s;LAYOUT %s;PORT %s;",
            hardcopyFormat[format], hardcopyLayout[layout],
            hardcopyPort[port]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;
    if (clearMenus != 0)  {
        Fmt (wrtBuf, "%s<CLEARMENU;");
        if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
            return TKTDS3xx_status;
    }
    Fmt (wrtBuf, "%s<HARDCOPY START;");
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;
    if (!port)  {
        /*  if GPIB save to file   */
        if (TKTDS3xx_readToFile (instrSession, fileName, 1e9, &retCnt) < 0)
            return TKTDS3xx_status;
    }
    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function:     Signal Path Cal                                           */
/* Description:  This function performs a self calibration of the signal   */
/*               path.                                                     */
/*=========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_signalPathCal (ViSession instrSession)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];

    Fmt (wrtBuf, "%s<*CAL?;");
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)
        return TKTDS3xx_status;
    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0)
        return TKTDS3xx_status;
    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    
    if ((TKTDS3xx_status = viPrintf (instrSession, "%s", writeBuffer)) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_readInstrData (ViSession instrSession, ViInt16 numBytes,
                    ViChar rdBuf[], ViPInt32 bytesRead)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    *bytesRead = 0L;
        
    if ((TKTDS3xx_status = viRead (instrSession, rdBuf, numBytes, bytesRead)) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus TKTDS3xx_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((TKTDS3xx_status = viWrite (instrSession, "*RST", 4, &retCnt)) < 0)
        return TKTDS3xx_status;

    if ((TKTDS3xx_status = TKTDS3xx_defaultInstrSetup (instrSession)) < 0)  
        return TKTDS3xx_status;
       
    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_selfTest (ViSession instrSession, ViPInt16 testResult)
{
    ViUInt32 retCnt = 0;
    ViStatus TKTDS3xx_status = VI_SUCCESS;

    if ((TKTDS3xx_status = viWrite (instrSession, "*TST?", 5, &retCnt)) < 0)
        return TKTDS3xx_status;

    if ((TKTDS3xx_status = viScanf (instrSession, "%hd", testResult)) < 0)
        return TKTDS3xx_status;
    
    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the  */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_errorQuery (ViSession instrSession, ViPInt32 errCode,
                    ViChar errMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus TKTDS3xx_status = VI_SUCCESS;

    if ((TKTDS3xx_status = viWrite (instrSession, "EVM?;", 5, &retCnt)) < 0)
        return TKTDS3xx_status;

    if ((TKTDS3xx_status = viScanf (instrSession, "%ld,\"%[^\"]", errCode, errMessage)) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_errorMessage (ViSession instrSession, ViStatus errorCode,
                    ViChar errMessage[])
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViInt16 i;
    static TKTDS3xx_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_PARAMETER9,   "ERROR: Parameter 9 out of range"},
        {VI_ERROR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_FILE_CLOSE,   "ERROR: Error closing the specified file"},
        {VI_ERROR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_REF_NOT_EXIST, "ERROR: Failed to select the specified waveform."},
        {VI_ERROR_FILE_EXISTS,  "ERROR: File already exists"},
        {VI_NULL, VI_NULL}
    };

    TKTDS3xx_status = viStatusDesc (instrSession, errorCode, errMessage);
    if (TKTDS3xx_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                strcpy (errMessage, statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        sprintf (errMessage, "Unknown Error 0x%08lX", errorCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    TKTDS3xx_status = VI_SUCCESS;
    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_revisionQuery (ViSession instrSession,
                    ViChar driverRev[], ViChar instrRev[])
{
    ViUInt32 retCnt = 0;
    ViStatus TKTDS3xx_status = VI_SUCCESS;

    if ((TKTDS3xx_status = viWrite (instrSession, "*IDN?", 5, &retCnt)) < 0)
        return TKTDS3xx_status;

    if ((TKTDS3xx_status = viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev)) < 0)
        return TKTDS3xx_status;

    strcpy (driverRev, TKTDS3xx_REVISION);
    
    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC TKTDS3xx_close (ViSession instrSession)
{
    ViSession rmSession;
    ViStatus TKTDS3xx_status = VI_SUCCESS;

    if ((TKTDS3xx_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return TKTDS3xx_status;
    
    TKTDS3xx_status = viClose (instrSession);
    viClose (rmSession);

    return TKTDS3xx_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*=========================================================================*/
/* Function: Waveform Selected                                             */
/* Purpose:  This function query the instrument to see if a waveform is    */
/*           selected or not.                                              */
/*=========================================================================*/
ViStatus TKTDS3xx_wvfmSelected (ViSession instrSession, ViInt16 source)
{
    ViInt16 result = -1, resp = 0;
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];    
    ViByte rdBuf[BUFFER_SIZE];

    Fmt (wrtBuf, ":SEL:%s?;", waveformSource[source]);
    if ((TKTDS3xx_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return TKTDS3xx_status;

    while (retCnt <= 1)
        if ((TKTDS3xx_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
            return TKTDS3xx_status;

    if (viScanf (instrSession, "%hd", &result) < 0) {
        TKTDS3xx_status = VI_ERROR_INTERPRETING_RESPONSE;
        result = -1;
        return result;
    }
    return result;
}

/*=========================================================================*/
/* Function Name:     Check Instrument Status                              */
/* Description:       Checks the status of the Instrument                  */
/*                    and sets TKTDS3xx Error accordingly.                 */
/* Return Value:      TKTDS3xx Error (Instrument Driver Status)            */
/*=========================================================================*/
ViStatus TKTDS3xx_checkInstrStatus (ViSession instrSession)
{
    ViInt16 event_status = 0;
    ViUInt32 retCnt = 0;
    ViByte wrtBuf[BUFFER_SIZE];    
    ViStatus  TKTDS3xx_status = VI_SUCCESS;

/*  Query the instrument for the Event Status Register contents.  */

    if ((TKTDS3xx_status = viWrite (instrSession, "*ESR?;", 6, &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = viScanf(instrSession, "%hd", &event_status) < 0)
        return TKTDS3xx_status;

    if ((event_status & 0x3c) != 0)
        TKTDS3xx_status = VI_ERROR_INV_RESPONSE;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Serial Poll                                                   */
/* Purpose:  This function performs a serial poll on the instrument.       */
/*           The status byte of the instrument is placed in the response   */
/*           variable. The return value is equal to the global error       */
/*           variable.                                                     */
/*  NOTE: Added a 55 mSec delay to this function because when the          */
/*        instrument driver is compiled and run as an executable program   */
/*        on a computer with a 486 33MHz processor, the serial poll        */
/*        function crashes the instrument when the serial poll is executed */
/*        in a loop. If you are running a slower machine or are not running*/
/*        an executable you may be able to remove the delay without        */
/*        adversely effecting your application.                            */
/*=========================================================================*/
ViStatus TKTDS3xx_poll (ViSession instrSession, ViUInt16 *response)
{
    ViStatus  TKTDS3xx_status = VI_SUCCESS;
    ViEvent event;
    ViInt32 timeout;

    Delay(.055);
    
    if (TKTDS3xx_status = viReadSTB (instrSession, response))
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Wait for OPC                                                  */
/* Purpose:  This function waits for the operation to complete.            */
/*=========================================================================*/
ViStatus TKTDS3xx_waitOpc (ViSession instrSession)
{
    ViStatus  TKTDS3xx_status = VI_SUCCESS;
    ViUInt16 resp = 0;
    ViUInt32 retCnt = 0;

    if ((TKTDS3xx_status = viWrite (instrSession, "*OPC;", 5, &retCnt)) < 0)
        return TKTDS3xx_status;

    while (!resp) {
        Delay(.055);
        if (TKTDS3xx_poll (instrSession, &resp) < 0)
            return TKTDS3xx_status;
    }
    return TKTDS3xx_status;
}

/*=========================================================================*/
/* Function: Clear OPC                                                     */
/* Purpose:  This function clears the operation complete register.         */
/*=========================================================================*/
ViStatus TKTDS3xx_clearOpc (ViSession instrSession)
{
    ViStatus  TKTDS3xx_status = VI_SUCCESS;
    ViChar rdBuf[BUFFER_SIZE];
    ViUInt32 retCnt = 0;

    if ((TKTDS3xx_status = viWrite (instrSession, "*ESR?;", 6, &retCnt)) < 0)
        return TKTDS3xx_status;

    if ((TKTDS3xx_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean TKTDS3xx_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the TKTDS3xx_init function.  When */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from TKTDS3xx_init to     */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus TKTDS3xx_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*=========================================================================*/
/* Function: File On Disk                                                  */
/* Purpose:  This function checks if a file exist or not.                  */
/*=========================================================================*/
ViBoolean TKTDS3xx_fileOnDisk (ViChar *file_name)
{
    ViInt16 fileHandle;

    if ((fileHandle = OpenFile (file_name, 1, 1, 1)) == -1)
        return 0;
    else {
        CloseFile (fileHandle);
        return 1;
    }
}

/*===========================================================================*/
/* Function: Read To File From Instrument                                    */
/* Purpose:  This function is used to read data from the instrument and      */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus TKTDS3xx_readToFile (ViSession instrSession, ViString filename,
                    ViUInt32 readBytes, ViPUInt32 retCount)
{
    ViStatus  TKTDS3xx_status = VI_SUCCESS;
    ViByte    buffer[BUFFER_SIZE];
    ViUInt32  bytesReadInstr = 0, bytesWrittenFile = 0;
    FILE     *targetFile;

    *retCount = 0L;
    if ((targetFile = fopen (filename, "wb")) == VI_NULL)
        return VI_ERROR_FILE_OPEN; /* not defined by VTL */

    for (;;) {
        if (readBytes > BUFFER_SIZE)
            TKTDS3xx_status = viRead (instrSession, buffer, BUFFER_SIZE, &bytesReadInstr);
        else
            TKTDS3xx_status = viRead (instrSession, buffer, readBytes, &bytesReadInstr);

        bytesWrittenFile = fwrite (buffer, sizeof (ViByte), (size_t)bytesReadInstr, targetFile);
        *retCount += bytesWrittenFile;
        if (bytesWrittenFile < bytesReadInstr)
            TKTDS3xx_status = VI_ERROR_FILE_WRITE; /* not defined by VTL */

        if ((readBytes <= BUFFER_SIZE) || (TKTDS3xx_status <= 0) || (TKTDS3xx_status == VI_SUCCESS_TERM_CHAR))
            break;

        readBytes -= BUFFER_SIZE;
    }

    fclose (targetFile);
    return TKTDS3xx_status;
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus TKTDS3xx_writeFromFile (ViSession instrSession, ViString filename,
                    ViUInt32 writeBytes, ViPUInt32 retCount)
{
    ViStatus  TKTDS3xx_status = VI_SUCCESS;
    ViByte    buffer[BUFFER_SIZE];
    ViUInt32  bytesRead = 0, bytesWritten = 0;
    FILE     *sourceFile;
    ViBoolean sendEnd = VI_FALSE;

    *retCount = 0L;
    if ((sourceFile = fopen (filename, "rb")) == VI_NULL)
        return VI_ERROR_FILE_OPEN; /* not defined by VTL */

    while (!feof (sourceFile)) {
        bytesRead = (ViUInt32)fread (buffer, sizeof (ViByte), BUFFER_SIZE, sourceFile);
        if ((writeBytes > BUFFER_SIZE) && (bytesRead == BUFFER_SIZE)) {
            viGetAttribute (instrSession, VI_ATTR_SEND_END_EN, &sendEnd);
            viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE);
            TKTDS3xx_status = viWrite (instrSession, buffer, BUFFER_SIZE, &bytesWritten);
            viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, sendEnd);
            writeBytes -= BUFFER_SIZE;
            *retCount += bytesWritten;
            if (TKTDS3xx_status < 0)
                break;
        }
        else {
            TKTDS3xx_status = viWrite (instrSession, buffer, ((bytesRead < writeBytes) ? bytesRead : writeBytes), &bytesWritten);
            *retCount += bytesWritten;
            break;
        }
    }

    fclose (sourceFile);
    return TKTDS3xx_status;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the TKTDS3xx_reset operation and by the   */
/*           TKTDS3xx_init function if the reset option has not been         */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus TKTDS3xx_defaultInstrSetup (ViSession instrSession)
{
    ViStatus TKTDS3xx_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViChar wrtBuff[256];
    
    if ((TKTDS3xx_status = viWrite (instrSession, ":HEADERS OFF;VERBOSE OFF;", 25, &retCnt)) < 0)
        return TKTDS3xx_status;
    /*  report all events  */

    if ((TKTDS3xx_status = viWrite (instrSession, "*CLS;DESE 61;*ESE 1;*SRE 0;", 27, &retCnt)) < 0)
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_waitOpc (instrSession) < 0)  
        return TKTDS3xx_status;

    if (TKTDS3xx_status = TKTDS3xx_clearOpc (instrSession) < 0) 
        return TKTDS3xx_status;

    return TKTDS3xx_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
